@@ -31,6 +31,9 @@ from pathlib import Path
 
 import typer
 from loguru import logger
+
+# Import Platform enum to filter excluded platforms
+from py_mcp_installer import Platform
 from rich.console import Console
 from rich.panel import Panel
 
@@ -53,6 +56,9 @@ from ..output import (
 # Import functions from refactored install module
 from .install import _install_to_platform, detect_all_platforms
 
+# Platforms to exclude from auto-setup (user can still manually install)
+EXCLUDED_PLATFORMS_FROM_SETUP = {Platform.CLAUDE_DESKTOP}
+
 # Create console for rich output
 console = Console()
 
@@ -959,15 +965,35 @@ async def _run_smart_setup(
     detected_platforms_list = detect_all_platforms()
 
     if detected_platforms_list:
-        platform_names = [p.platform.value for p in detected_platforms_list]
-        print_success(
-            f"   ✅ Found {len(platform_names)} platform(s): {', '.join(platform_names)}"
-        )
-        if verbose:
-            for platform_info in detected_platforms_list:
-                print_info(
-                    f"      {platform_info.platform.value}: {platform_info.config_path}"
-                )
+        # Filter out excluded platforms for display
+        configurable_platforms = [
+            p
+            for p in detected_platforms_list
+            if p.platform not in EXCLUDED_PLATFORMS_FROM_SETUP
+        ]
+        excluded_platforms = [
+            p
+            for p in detected_platforms_list
+            if p.platform in EXCLUDED_PLATFORMS_FROM_SETUP
+        ]
+
+        if configurable_platforms:
+            platform_names = [p.platform.value for p in configurable_platforms]
+            print_success(
+                f"   ✅ Found {len(platform_names)} platform(s): {', '.join(platform_names)}"
+            )
+            if verbose:
+                for platform_info in configurable_platforms:
+                    print_info(
+                        f"      {platform_info.platform.value}: {platform_info.config_path}"
+                    )
+
+        # Note excluded platforms
+        if excluded_platforms:
+            excluded_names = [p.platform.value for p in excluded_platforms]
+            print_info(
+                f"   ℹ️  Skipping: {', '.join(excluded_names)} (use 'install mcp --platform' for manual install)"
+            )
     else:
         print_info("   No MCP platforms detected (will configure Claude Code)")
 
@@ -1012,7 +1038,27 @@ async def _run_smart_setup(
     # ===========================================================================
     # Phase 4: Indexing
     # ===========================================================================
-    if not already_initialized or force:
+    # Determine if indexing is needed:
+    # 1. Not already initialized (new setup)
+    # 2. Force flag is set
+    # 3. Index database doesn't exist
+    # 4. Index exists but is empty
+    # 5. Files have changed (incremental indexing will handle this)
+    needs_indexing = not already_initialized or force
+
+    if already_initialized and not force:
+        # Check if index exists and has content
+        index_db_path = project_root / ".mcp-vector-search" / "chroma.sqlite3"
+        if not index_db_path.exists():
+            print_info("   Index database not found, will create...")
+            needs_indexing = True
+        else:
+            # Check if index is empty or files have changed
+            # Run incremental indexing to catch any changes
+            print_info("   Checking for file changes...")
+            needs_indexing = True  # Always run incremental to catch changes
+
+    if needs_indexing:
         console.print("\n[bold blue]🔍 Indexing codebase...[/bold blue]")
 
         from .index import run_indexing
@@ -1045,7 +1091,12 @@ async def _run_smart_setup(
         print_info("   ✅ Claude CLI detected, using native integration")
 
     # Use detected platforms or default to empty list
-    platforms_to_configure = detected_platforms_list if detected_platforms_list else []
+    # Filter out excluded platforms (e.g., Claude Desktop) - exclusion already noted in Phase 1
+    platforms_to_configure = [
+        p
+        for p in (detected_platforms_list or [])
+        if p.platform not in EXCLUDED_PLATFORMS_FROM_SETUP
+    ]
 
     # Configure all detected platforms using new library
     for platform_info in platforms_to_configure:
